"use strict";Object.defineProperty(exports, "__esModule", {value: true}); function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; } function _optionalChain(ops) { let lastAccessLHS = undefined; let value = ops[0]; let i = 1; while (i < ops.length) { const op = ops[i]; const fn = ops[i + 1]; i += 2; if ((op === 'optionalAccess' || op === 'optionalCall') && value == null) { return undefined; } if (op === 'access' || op === 'optionalAccess') { lastAccessLHS = value; value = fn(value); } else if (op === 'call' || op === 'optionalCall') { value = fn((...args) => value.call(lastAccessLHS, ...args)); lastAccessLHS = undefined; } } return value; }// src/index.ts
var _vite = require('vite');

// src/createMockServer.ts
var _fs = require('fs');
var _path = require('path'); var _path2 = _interopRequireDefault(_path);
var _chokidar = require('chokidar'); var _chokidar2 = _interopRequireDefault(_chokidar);
var _chalk = require('chalk'); var _chalk2 = _interopRequireDefault(_chalk);
var _url = require('url'); var _url2 = _interopRequireDefault(_url);
var _fastglob = require('fast-glob'); var _fastglob2 = _interopRequireDefault(_fastglob);
var _mockjs = require('mockjs'); var _mockjs2 = _interopRequireDefault(_mockjs);

// src/utils.ts

var toString = Object.prototype.toString;
function is(val, type) {
  return toString.call(val) === `[object ${type}]`;
}
function isFunction(val) {
  return is(val, "Function");
}
function isArray(val) {
  return val && Array.isArray(val);
}
function isRegExp(val) {
  return is(val, "RegExp");
}
function isBoolean(val) {
  return is(val, "Boolean");
}
function sleep(time) {
  return new Promise((resolve) => {
    setTimeout(() => {
      resolve("");
    }, time);
  });
}
async function loadConfigFromBundledFile(fileName, bundledCode) {
  const extension = _path2.default.extname(fileName);
  const defaultLoader = require.extensions[extension];
  require.extensions[extension] = (module, filename) => {
    if (filename === fileName) {
      module._compile(bundledCode, filename);
    } else {
      defaultLoader(module, filename);
    }
  };
  delete require.cache[fileName];
  const raw = require(fileName);
  const config = raw.__esModule ? raw.default : raw;
  require.extensions[extension] = defaultLoader;
  return config;
}

// src/createMockServer.ts
var _rollup = require('rollup');
var _rolluppluginesbuild = require('rollup-plugin-esbuild'); var _rolluppluginesbuild2 = _interopRequireDefault(_rolluppluginesbuild);
var _dayjs = require('dayjs'); var _dayjs2 = _interopRequireDefault(_dayjs);
var pathResolve = require("@rollup/plugin-node-resolve");
var mockData = [];
async function createMockServer(opt = {mockPath: "mock", ignoreFiles: [], configPath: "vite.mock.config"}) {
  opt = {
    mockPath: "mock",
    ignoreFiles: [],
    watchFiles: true,
    supportTs: true,
    configPath: "vite.mock.config.ts",
    ...opt
  };
  if (mockData.length > 0)
    return;
  mockData = await getMockConfig(opt);
  const {watchFiles = true} = opt;
  if (watchFiles) {
    const watch = await createWatch(opt);
    watch && watch();
  }
}
function getInvokeTime(opt) {
  const {showTime} = opt;
  const defTime = ` - ${_dayjs2.default.call(void 0, ).format("YYYY-MM-DD HH:mm:ss")}`;
  if (isBoolean(showTime)) {
    return !showTime ? "" : defTime;
  }
  if (!showTime)
    return "";
  try {
    return _dayjs2.default.call(void 0, ).format(showTime);
  } catch (error) {
    return defTime;
  }
}
function requestMiddle(opt) {
  const middleware = async (req, res, next) => {
    let queryParams = {};
    const isGet = req.method === "GET";
    if (isGet && req.url) {
      queryParams = _url2.default.parse(req.url, true);
    }
    const matchReq = mockData.find((item) => {
      if (isGet) {
        return item.url === queryParams.pathname;
      }
      return item.url === req.url;
    });
    if (matchReq) {
      const {response, timeout, statusCode} = matchReq;
      if (timeout) {
        await sleep(timeout);
      }
      const {query = {}} = queryParams;
      const body = req.body;
      const mockRes = isFunction(response) ? response({body, query}) : response;
      console.log(`${_chalk2.default.green(`[vite:mock-server${getInvokeTime(opt)}]:request invoke:  ${_chalk2.default.cyan(req.url)} `)}`);
      res.setHeader("Content-Type", "application/json");
      res.statusCode = statusCode || 200;
      res.end(JSON.stringify(_mockjs2.default.mock(mockRes)));
      return;
    }
    next();
  };
  return middleware;
}
function createWatch(opt) {
  const {configPath} = opt;
  const {absConfigPath, absMockPath} = getPath(opt);
  if (process.env.VITE_DISABLED_WATCH_MOCK === "true")
    return;
  const watchDir = [];
  const exitsConfigPath = _fs.existsSync.call(void 0, absConfigPath);
  exitsConfigPath && configPath ? watchDir.push(absConfigPath) : watchDir.push(absMockPath);
  const watcher = _chokidar2.default.watch(watchDir, {
    ignoreInitial: true
  });
  const watch = () => {
    watcher.on("all", async (event, file) => {
      console.log(_chalk2.default.magentaBright(`[vite:mock-server]:mock file ${event}.`));
      mockData = await getMockConfig(opt);
    });
  };
  return watch;
}
function cleanRequireCache(opt) {
  const {absConfigPath, absMockPath} = getPath(opt);
  Object.keys(require.cache).forEach((file) => {
    if (file === absConfigPath || file.indexOf(absMockPath) > -1) {
      delete require.cache[file];
    }
  });
}
async function getMockConfig(opt) {
  cleanRequireCache(opt);
  const {absConfigPath, absMockPath} = getPath(opt);
  const {ignoreFiles = [], ignore, configPath, supportTs} = opt;
  let ret = [];
  if (configPath && _fs.existsSync.call(void 0, absConfigPath)) {
    console.log(_chalk2.default.blue(`[vite:mock-server]:load mock data from ${absConfigPath}`));
    let resultModule = await resolveModule(absConfigPath);
    ret = resultModule;
  } else {
    const mockFiles = _fastglob2.default.sync(`**/*.${supportTs ? "ts" : "js"}`, {
      cwd: absMockPath,
      ignore: ignoreFiles
    }).filter((item) => {
      if (!ignore) {
        return true;
      }
      if (isFunction(ignore)) {
        return ignore(item);
      }
      if (isRegExp(ignore)) {
        return !ignore.test(item);
      }
      return true;
    });
    try {
      ret = [];
      for (let index = 0; index < mockFiles.length; index++) {
        const mockFile = mockFiles[index];
        const resultModule = await resolveModule(_path.join.call(void 0, absMockPath, mockFile));
        let mod = resultModule;
        if (!isArray(mod)) {
          mod = [mod];
        }
        ret = [...ret, ...mod];
      }
    } catch (error) {
      console.log(`${_chalk2.default.red("[vite:mock-server]:mock reload error!")}
${error}`);
      ret = [];
    }
  }
  return ret;
}
async function resolveModule(path3) {
  const nodeResolve = pathResolve.nodeResolve({
    extensions: [".js", ".ts"]
  });
  const bundle = await _rollup.rollup.call(void 0, {
    input: path3,
    treeshake: false,
    plugins: [
      _rolluppluginesbuild2.default.call(void 0, {
        include: /\.[jt]sx?$/,
        exclude: /node_modules/,
        sourceMap: false
      }),
      nodeResolve
    ]
  });
  const {
    output: [{code}]
  } = await bundle.generate({
    exports: "named",
    format: "cjs"
  });
  return await loadConfigFromBundledFile(path3, code);
}
function getPath(opt) {
  const {mockPath, configPath} = opt;
  const cwd = process.cwd();
  const absMockPath = _path.join.call(void 0, cwd, mockPath || "");
  const absConfigPath = _path.join.call(void 0, cwd, configPath || "");
  return {
    absMockPath,
    absConfigPath
  };
}

// src/index.ts
var _bodyparser = require('body-parser'); var _bodyparser2 = _interopRequireDefault(_bodyparser);

var _core = require('@babel/core'); var _core2 = _interopRequireDefault(_core);
function viteMockServe(opt) {
  const {supportTs = true} = opt;
  const {
    injectFile = _vite.normalizePath.call(void 0, _path2.default.resolve(process.cwd(), `src/main.${supportTs ? "ts" : "js"}`))
  } = opt;
  let isDev = false;
  let config;
  let needSourceMap = false;
  return {
    name: "vite:mock",
    enforce: "pre",
    configResolved(resolvedConfig) {
      config = resolvedConfig;
      isDev = config.command === "serve" && !config.isProduction;
      needSourceMap = config.isProduction && !!config.build.sourcemap;
    },
    configureServer: ({middlewares}) => {
      const {localEnabled = isDev} = opt;
      if (!localEnabled)
        return;
      createMockServer(opt);
      middlewares.use(_bodyparser2.default.urlencoded({extended: false}));
      middlewares.use(_bodyparser2.default.json());
      middlewares.use(requestMiddle(opt));
    },
    async transform(code, id) {
      let map = null;
      if (needSourceMap) {
        const plugins = [];
        if (id.endsWith(".tsx")) {
          plugins.push([
            require("@babel/plugin-transform-typescript"),
            {isTSX: true, allowExtensions: true}
          ]);
        }
        const result = _core2.default.transformSync(code, {
          ast: false,
          plugins,
          sourceMaps: true,
          sourceFileName: id
        });
        map = _optionalChain([result, 'optionalAccess', _ => _.map]);
      }
      const defResult = {
        code,
        map
      };
      if (isDev || !id.endsWith(injectFile))
        return defResult;
      const {prodEnabled = true, injectCode = ""} = opt;
      if (!prodEnabled)
        return defResult;
      return {
        code: `
      ${code}
      

      ${injectCode}
      `,
        map
      };
    }
  };
}


exports.viteMockServe = viteMockServe;
